import { Logger } from "winston"

import NetToolBase, { NetError } from "../netTool/base"
import { Lark } from "../types"
export class LarkAuthService extends NetToolBase {
  public appId: string
  public appSecret: string

  constructor({
    appId,
    appSecret,
    logger,
  }: {
    appId?: string
    appSecret?: string
    logger?: Logger
  }) {
    super({
      prefix:
        process.env.NODE_ENV === "production"
          ? "http://lark-auth.c5-cloudml.xiaomi.srv/lark_auth"
          : "http://lark-auth.staging-cloudml.xiaomi.srv/lark_auth",
      logger,
    })
    this.appId = appId || process.env.LARK_APP_ID || ""
    this.appSecret = appSecret || process.env.LARK_APP_SECRET || ""
  }

  /**
   * 获取用户的授权信息
   * @param user
   * @returns
   */
  public async getUserAuth(user: string) {
    return this.get<
      Lark.BaseRes<{
        access_token: string
      }>
    >("/get_auth", {
      app_id: this.appId,
      app_secret: this.appSecret,
      user_name: user,
    })
      .then(({ data: { access_token } }) => ({
        access_token,
      }))
      .catch((e: NetError) => {
        if (e.response?.status === 400)
          return {
            auth_url: e.data?.detail?.auth_url,
          }
        throw e
      })
  }

  /**
   * 删除用户的授权信息
   * @param user
   * @returns
   */
  public deleteUserAuth(user: string) {
    return this.get<{ message: string }>("/delete_auth", {
      app_id: this.appId,
      app_secret: this.appSecret,
      user_name: user,
    })
  }

  /**
   * 获取应用的授权信息
   * @returns
   */
  public async getAppAuth() {
    const {
      data: { tenant_access_token },
    } = await this.get<Lark.BaseRes<{ tenant_access_token: string }>>(
      "/get_tenant_access_token",
      {
        app_id: this.appId,
        app_secret: this.appSecret,
      }
    )
    return tenant_access_token
  }
}

export default LarkAuthService
